{% extends "base.html" %} {% block title %}Welcome{% endblock %} {% block content %}
<h1>Welcome to Turbines!</h1>
<p>This is the index.html page. You can start building your application from here.</p>

<p>Try going to <a href="/test.html">this</a> page which is generated from markdown.</p>

{% endblock %}
