<html>
    <head>
        <title>{{ title }}</title>
        <link rel="stylesheet" href="/static/style.css" />
    </head>
    <body>
        <header>
            <p>{{ site.title }}</p>
        </header>
        <main>{% block content %}{% endblock %}</main>
        <footer>
            {% block footer %}
            <p>&copy; {{context.copyright_year}}</p>
            {% endblock %}
        </footer>
    </body>
</html>
