body {
    margin: 0;
    padding: 0;
    font-family: Arial, sans-serif;
    background: #fafbfc;
    color: #222;
}

main {
    padding: 1rem;
    max-width: 80rem;
    margin: auto;
}

header {
    background: #2c3e50;
    color: #fff;
    padding: 0.5rem 2rem;
    font-weight: bold;
    text-align: left;
    font-size: 1.5rem;
}

footer {
    background: #2c3e50;
    color: #fff;
    padding: 0.75rem 2rem;
    text-align: center;
    font-size: 1rem;
    width: 100%;
    box-sizing: border-box;
}
